/* valalockable.c generated by valac, the Vala compiler
 * generated from valalockable.vala, do not modify */

/* valalockable.vala
 *
 * Copyright (C) 2006-2007  Raffaele Sandrini, Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Raffaele Sandrini <raffaele@sandrini.ch>
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "vala.h"
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static GType vala_lockable_get_type_once (void);

gboolean
vala_lockable_get_lock_used (ValaLockable* self)
{
	ValaLockableIface* _iface_;
	g_return_val_if_fail (self != NULL, FALSE);
	_iface_ = VALA_LOCKABLE_GET_INTERFACE (self);
	if (_iface_->get_lock_used) {
		return _iface_->get_lock_used (self);
	}
	return FALSE;
}

void
vala_lockable_set_lock_used (ValaLockable* self,
                             gboolean value)
{
	ValaLockableIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = VALA_LOCKABLE_GET_INTERFACE (self);
	if (_iface_->set_lock_used) {
		_iface_->set_lock_used (self, value);
	}
}

static void
vala_lockable_default_init (ValaLockableIface * iface,
                            gpointer iface_data)
{
}

/**
 * Represents a lockable object.
 */
static GType
vala_lockable_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValaLockableIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_lockable_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType vala_lockable_type_id;
	vala_lockable_type_id = g_type_register_static (G_TYPE_INTERFACE, "ValaLockable", &g_define_type_info, 0);
	return vala_lockable_type_id;
}

GType
vala_lockable_get_type (void)
{
	static volatile gsize vala_lockable_type_id__once = 0;
	if (g_once_init_enter (&vala_lockable_type_id__once)) {
		GType vala_lockable_type_id;
		vala_lockable_type_id = vala_lockable_get_type_once ();
		g_once_init_leave (&vala_lockable_type_id__once, vala_lockable_type_id);
	}
	return vala_lockable_type_id__once;
}

