/* valaccodecontinuestatement.c generated by valac, the Vala compiler
 * generated from valaccodecontinuestatement.vala, do not modify */

/* valaccodecontinuestatement.vala
 *
 * Copyright (C) 2006-2007  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "valaccode.h"
#include <glib.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static gpointer vala_ccode_continue_statement_parent_class = NULL;

static void vala_ccode_continue_statement_real_write (ValaCCodeNode* base,
                                               ValaCCodeWriter* writer);
static GType vala_ccode_continue_statement_get_type_once (void);

static void
vala_ccode_continue_statement_real_write (ValaCCodeNode* base,
                                          ValaCCodeWriter* writer)
{
	ValaCCodeContinueStatement * self;
	ValaCCodeLineDirective* _tmp0_;
	ValaCCodeLineDirective* _tmp1_;
	self = (ValaCCodeContinueStatement*) base;
	g_return_if_fail (writer != NULL);
	_tmp0_ = vala_ccode_node_get_line ((ValaCCodeNode*) self);
	_tmp1_ = _tmp0_;
	vala_ccode_writer_write_indent (writer, _tmp1_);
	vala_ccode_writer_write_string (writer, "continue;");
	vala_ccode_writer_write_newline (writer);
}

ValaCCodeContinueStatement*
vala_ccode_continue_statement_construct (GType object_type)
{
	ValaCCodeContinueStatement* self = NULL;
	self = (ValaCCodeContinueStatement*) vala_ccode_statement_construct (object_type);
	return self;
}

ValaCCodeContinueStatement*
vala_ccode_continue_statement_new (void)
{
	return vala_ccode_continue_statement_construct (VALA_TYPE_CCODE_CONTINUE_STATEMENT);
}

static void
vala_ccode_continue_statement_class_init (ValaCCodeContinueStatementClass * klass,
                                          gpointer klass_data)
{
	vala_ccode_continue_statement_parent_class = g_type_class_peek_parent (klass);
	((ValaCCodeNodeClass *) klass)->write = (void (*) (ValaCCodeNode*, ValaCCodeWriter*)) vala_ccode_continue_statement_real_write;
}

static void
vala_ccode_continue_statement_instance_init (ValaCCodeContinueStatement * self,
                                             gpointer klass)
{
}

/**
 * Represents a continue statement in the C code.
 */
static GType
vala_ccode_continue_statement_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeContinueStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_continue_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeContinueStatement), 0, (GInstanceInitFunc) vala_ccode_continue_statement_instance_init, NULL };
	GType vala_ccode_continue_statement_type_id;
	vala_ccode_continue_statement_type_id = g_type_register_static (VALA_TYPE_CCODE_STATEMENT, "ValaCCodeContinueStatement", &g_define_type_info, 0);
	return vala_ccode_continue_statement_type_id;
}

GType
vala_ccode_continue_statement_get_type (void)
{
	static volatile gsize vala_ccode_continue_statement_type_id__once = 0;
	if (g_once_init_enter (&vala_ccode_continue_statement_type_id__once)) {
		GType vala_ccode_continue_statement_type_id;
		vala_ccode_continue_statement_type_id = vala_ccode_continue_statement_get_type_once ();
		g_once_init_leave (&vala_ccode_continue_statement_type_id__once, vala_ccode_continue_statement_type_id);
	}
	return vala_ccode_continue_statement_type_id__once;
}

