/* valaaddressofexpression.c generated by valac, the Vala compiler
 * generated from valaaddressofexpression.vala, do not modify */

/* valaaddressofexpression.vala
 *
 * Copyright (C) 2007-2010  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "vala.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))

struct _ValaAddressofExpressionPrivate {
	ValaExpression* _inner;
};

static gint ValaAddressofExpression_private_offset;
static gpointer vala_addressof_expression_parent_class = NULL;

static void vala_addressof_expression_set_inner (ValaAddressofExpression* self,
                                          ValaExpression* value);
static void vala_addressof_expression_real_accept (ValaCodeNode* base,
                                            ValaCodeVisitor* visitor);
static void vala_addressof_expression_real_accept_children (ValaCodeNode* base,
                                                     ValaCodeVisitor* visitor);
static gchar* vala_addressof_expression_real_to_string (ValaCodeNode* base);
static void vala_addressof_expression_real_replace_expression (ValaCodeNode* base,
                                                        ValaExpression* old_node,
                                                        ValaExpression* new_node);
static gboolean vala_addressof_expression_real_is_pure (ValaExpression* base);
static gboolean vala_addressof_expression_real_is_accessible (ValaExpression* base,
                                                       ValaSymbol* sym);
static gboolean vala_addressof_expression_real_is_non_null (ValaExpression* base);
static gboolean vala_addressof_expression_real_check (ValaCodeNode* base,
                                               ValaCodeContext* context);
static void vala_addressof_expression_real_emit (ValaCodeNode* base,
                                          ValaCodeGenerator* codegen);
static void vala_addressof_expression_finalize (ValaCodeNode * obj);
static GType vala_addressof_expression_get_type_once (void);

static inline gpointer
vala_addressof_expression_get_instance_private (ValaAddressofExpression* self)
{
	return G_STRUCT_MEMBER_P (self, ValaAddressofExpression_private_offset);
}

ValaExpression*
vala_addressof_expression_get_inner (ValaAddressofExpression* self)
{
	ValaExpression* result;
	ValaExpression* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_inner;
	result = _tmp0_;
	return result;
}

static gpointer
_vala_code_node_ref0 (gpointer self)
{
	return self ? vala_code_node_ref (self) : NULL;
}

static void
vala_addressof_expression_set_inner (ValaAddressofExpression* self,
                                     ValaExpression* value)
{
	ValaExpression* _tmp0_;
	ValaExpression* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = _vala_code_node_ref0 (value);
	_vala_code_node_unref0 (self->priv->_inner);
	self->priv->_inner = _tmp0_;
	_tmp1_ = self->priv->_inner;
	vala_code_node_set_parent_node ((ValaCodeNode*) _tmp1_, (ValaCodeNode*) self);
}

/**
 * Creates a new address-of expression.
 *
 * @param inner variable whose address is to be computed
 * @return      newly created address-of expression
 */
ValaAddressofExpression*
vala_addressof_expression_construct (GType object_type,
                                     ValaExpression* inner,
                                     ValaSourceReference* source_reference)
{
	ValaAddressofExpression* self = NULL;
	g_return_val_if_fail (inner != NULL, NULL);
	self = (ValaAddressofExpression*) vala_expression_construct (object_type);
	vala_code_node_set_source_reference ((ValaCodeNode*) self, source_reference);
	vala_addressof_expression_set_inner (self, inner);
	return self;
}

ValaAddressofExpression*
vala_addressof_expression_new (ValaExpression* inner,
                               ValaSourceReference* source_reference)
{
	return vala_addressof_expression_construct (VALA_TYPE_ADDRESSOF_EXPRESSION, inner, source_reference);
}

static void
vala_addressof_expression_real_accept (ValaCodeNode* base,
                                       ValaCodeVisitor* visitor)
{
	ValaAddressofExpression * self;
	self = (ValaAddressofExpression*) base;
	g_return_if_fail (visitor != NULL);
	vala_code_visitor_visit_addressof_expression (visitor, self);
	vala_code_visitor_visit_expression (visitor, (ValaExpression*) self);
}

static void
vala_addressof_expression_real_accept_children (ValaCodeNode* base,
                                                ValaCodeVisitor* visitor)
{
	ValaAddressofExpression * self;
	ValaExpression* _tmp0_;
	ValaExpression* _tmp1_;
	self = (ValaAddressofExpression*) base;
	g_return_if_fail (visitor != NULL);
	_tmp0_ = vala_addressof_expression_get_inner (self);
	_tmp1_ = _tmp0_;
	vala_code_node_accept ((ValaCodeNode*) _tmp1_, visitor);
}

static gchar*
vala_addressof_expression_real_to_string (ValaCodeNode* base)
{
	ValaAddressofExpression * self;
	ValaExpression* _tmp0_;
	ValaExpression* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	gchar* result;
	self = (ValaAddressofExpression*) base;
	_tmp0_ = vala_addressof_expression_get_inner (self);
	_tmp1_ = _tmp0_;
	_tmp2_ = vala_code_node_to_string ((ValaCodeNode*) _tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = g_strdup_printf ("(&%s)", _tmp3_);
	_tmp5_ = _tmp4_;
	_g_free0 (_tmp3_);
	result = _tmp5_;
	return result;
}

static void
vala_addressof_expression_real_replace_expression (ValaCodeNode* base,
                                                   ValaExpression* old_node,
                                                   ValaExpression* new_node)
{
	ValaAddressofExpression * self;
	ValaExpression* _tmp0_;
	ValaExpression* _tmp1_;
	self = (ValaAddressofExpression*) base;
	g_return_if_fail (old_node != NULL);
	g_return_if_fail (new_node != NULL);
	_tmp0_ = vala_addressof_expression_get_inner (self);
	_tmp1_ = _tmp0_;
	if (_tmp1_ == old_node) {
		vala_addressof_expression_set_inner (self, new_node);
	}
}

static gboolean
vala_addressof_expression_real_is_pure (ValaExpression* base)
{
	ValaAddressofExpression * self;
	ValaExpression* _tmp0_;
	ValaExpression* _tmp1_;
	gboolean result;
	self = (ValaAddressofExpression*) base;
	_tmp0_ = vala_addressof_expression_get_inner (self);
	_tmp1_ = _tmp0_;
	result = vala_expression_is_pure (_tmp1_);
	return result;
}

static gboolean
vala_addressof_expression_real_is_accessible (ValaExpression* base,
                                              ValaSymbol* sym)
{
	ValaAddressofExpression * self;
	ValaExpression* _tmp0_;
	ValaExpression* _tmp1_;
	gboolean result;
	self = (ValaAddressofExpression*) base;
	g_return_val_if_fail (sym != NULL, FALSE);
	_tmp0_ = vala_addressof_expression_get_inner (self);
	_tmp1_ = _tmp0_;
	result = vala_expression_is_accessible (_tmp1_, sym);
	return result;
}

static gboolean
vala_addressof_expression_real_is_non_null (ValaExpression* base)
{
	ValaAddressofExpression * self;
	ValaExpression* _tmp0_;
	ValaExpression* _tmp1_;
	gboolean result;
	self = (ValaAddressofExpression*) base;
	_tmp0_ = vala_addressof_expression_get_inner (self);
	_tmp1_ = _tmp0_;
	result = vala_expression_is_non_null (_tmp1_);
	return result;
}

static gboolean
vala_addressof_expression_real_check (ValaCodeNode* base,
                                      ValaCodeContext* context)
{
	ValaAddressofExpression * self;
	gboolean _tmp0_;
	gboolean _tmp1_;
	ValaExpression* _tmp4_;
	ValaExpression* _tmp5_;
	ValaExpression* _tmp6_;
	ValaExpression* _tmp7_;
	ValaElementAccess* ea = NULL;
	ValaExpression* _tmp8_;
	ValaExpression* _tmp9_;
	gboolean _tmp10_ = FALSE;
	ValaExpression* _tmp11_;
	ValaExpression* _tmp12_;
	ValaExpression* _tmp32_;
	ValaExpression* _tmp33_;
	ValaDataType* _tmp34_;
	ValaDataType* _tmp35_;
	gboolean _tmp54_;
	gboolean _tmp55_;
	gboolean result;
	self = (ValaAddressofExpression*) base;
	g_return_val_if_fail (context != NULL, FALSE);
	_tmp0_ = vala_code_node_get_checked ((ValaCodeNode*) self);
	_tmp1_ = _tmp0_;
	if (_tmp1_) {
		gboolean _tmp2_;
		gboolean _tmp3_;
		_tmp2_ = vala_code_node_get_error ((ValaCodeNode*) self);
		_tmp3_ = _tmp2_;
		result = !_tmp3_;
		return result;
	}
	vala_code_node_set_checked ((ValaCodeNode*) self, TRUE);
	_tmp4_ = vala_addressof_expression_get_inner (self);
	_tmp5_ = _tmp4_;
	vala_expression_set_lvalue (_tmp5_, TRUE);
	_tmp6_ = vala_addressof_expression_get_inner (self);
	_tmp7_ = _tmp6_;
	if (!vala_code_node_check ((ValaCodeNode*) _tmp7_, context)) {
		vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
		result = FALSE;
		return result;
	}
	_tmp8_ = vala_addressof_expression_get_inner (self);
	_tmp9_ = _tmp8_;
	ea = VALA_IS_ELEMENT_ACCESS (_tmp9_) ? ((ValaElementAccess*) _tmp9_) : NULL;
	_tmp11_ = vala_addressof_expression_get_inner (self);
	_tmp12_ = _tmp11_;
	if (VALA_IS_MEMBER_ACCESS (_tmp12_)) {
		ValaExpression* _tmp13_;
		ValaExpression* _tmp14_;
		ValaSymbol* _tmp15_;
		ValaSymbol* _tmp16_;
		_tmp13_ = vala_addressof_expression_get_inner (self);
		_tmp14_ = _tmp13_;
		_tmp15_ = vala_expression_get_symbol_reference (_tmp14_);
		_tmp16_ = _tmp15_;
		_tmp10_ = VALA_IS_VARIABLE (_tmp16_);
	} else {
		_tmp10_ = FALSE;
	}
	if (_tmp10_) {
	} else {
		gboolean _tmp17_ = FALSE;
		ValaElementAccess* _tmp18_;
		_tmp18_ = ea;
		if (_tmp18_ != NULL) {
			gboolean _tmp19_ = FALSE;
			ValaElementAccess* _tmp20_;
			ValaExpression* _tmp21_;
			ValaExpression* _tmp22_;
			ValaDataType* _tmp23_;
			ValaDataType* _tmp24_;
			_tmp20_ = ea;
			_tmp21_ = vala_element_access_get_container (_tmp20_);
			_tmp22_ = _tmp21_;
			_tmp23_ = vala_expression_get_value_type (_tmp22_);
			_tmp24_ = _tmp23_;
			if (VALA_IS_ARRAY_TYPE (_tmp24_)) {
				_tmp19_ = TRUE;
			} else {
				ValaElementAccess* _tmp25_;
				ValaExpression* _tmp26_;
				ValaExpression* _tmp27_;
				ValaDataType* _tmp28_;
				ValaDataType* _tmp29_;
				_tmp25_ = ea;
				_tmp26_ = vala_element_access_get_container (_tmp25_);
				_tmp27_ = _tmp26_;
				_tmp28_ = vala_expression_get_value_type (_tmp27_);
				_tmp29_ = _tmp28_;
				_tmp19_ = VALA_IS_POINTER_TYPE (_tmp29_);
			}
			_tmp17_ = _tmp19_;
		} else {
			_tmp17_ = FALSE;
		}
		if (_tmp17_) {
		} else {
			ValaSourceReference* _tmp30_;
			ValaSourceReference* _tmp31_;
			vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
			_tmp30_ = vala_code_node_get_source_reference ((ValaCodeNode*) self);
			_tmp31_ = _tmp30_;
			vala_report_error (_tmp31_, "Address-of operator not supported for this expression");
			result = FALSE;
			return result;
		}
	}
	_tmp32_ = vala_addressof_expression_get_inner (self);
	_tmp33_ = _tmp32_;
	_tmp34_ = vala_expression_get_value_type (_tmp33_);
	_tmp35_ = _tmp34_;
	if (vala_data_type_is_reference_type_or_type_parameter (_tmp35_)) {
		ValaExpression* _tmp36_;
		ValaExpression* _tmp37_;
		ValaDataType* _tmp38_;
		ValaDataType* _tmp39_;
		ValaPointerType* _tmp40_;
		ValaPointerType* _tmp41_;
		ValaSourceReference* _tmp42_;
		ValaSourceReference* _tmp43_;
		ValaPointerType* _tmp44_;
		ValaPointerType* _tmp45_;
		_tmp36_ = vala_addressof_expression_get_inner (self);
		_tmp37_ = _tmp36_;
		_tmp38_ = vala_expression_get_value_type (_tmp37_);
		_tmp39_ = _tmp38_;
		_tmp40_ = vala_pointer_type_new (_tmp39_, NULL);
		_tmp41_ = _tmp40_;
		_tmp42_ = vala_code_node_get_source_reference ((ValaCodeNode*) self);
		_tmp43_ = _tmp42_;
		_tmp44_ = vala_pointer_type_new ((ValaDataType*) _tmp41_, _tmp43_);
		_tmp45_ = _tmp44_;
		vala_expression_set_value_type ((ValaExpression*) self, (ValaDataType*) _tmp45_);
		_vala_code_node_unref0 (_tmp45_);
		_vala_code_node_unref0 (_tmp41_);
	} else {
		ValaExpression* _tmp46_;
		ValaExpression* _tmp47_;
		ValaDataType* _tmp48_;
		ValaDataType* _tmp49_;
		ValaSourceReference* _tmp50_;
		ValaSourceReference* _tmp51_;
		ValaPointerType* _tmp52_;
		ValaPointerType* _tmp53_;
		_tmp46_ = vala_addressof_expression_get_inner (self);
		_tmp47_ = _tmp46_;
		_tmp48_ = vala_expression_get_value_type (_tmp47_);
		_tmp49_ = _tmp48_;
		_tmp50_ = vala_code_node_get_source_reference ((ValaCodeNode*) self);
		_tmp51_ = _tmp50_;
		_tmp52_ = vala_pointer_type_new (_tmp49_, _tmp51_);
		_tmp53_ = _tmp52_;
		vala_expression_set_value_type ((ValaExpression*) self, (ValaDataType*) _tmp53_);
		_vala_code_node_unref0 (_tmp53_);
	}
	_tmp54_ = vala_code_node_get_error ((ValaCodeNode*) self);
	_tmp55_ = _tmp54_;
	result = !_tmp55_;
	return result;
}

static void
vala_addressof_expression_real_emit (ValaCodeNode* base,
                                     ValaCodeGenerator* codegen)
{
	ValaAddressofExpression * self;
	ValaExpression* _tmp0_;
	ValaExpression* _tmp1_;
	self = (ValaAddressofExpression*) base;
	g_return_if_fail (codegen != NULL);
	_tmp0_ = vala_addressof_expression_get_inner (self);
	_tmp1_ = _tmp0_;
	vala_code_node_emit ((ValaCodeNode*) _tmp1_, codegen);
	vala_code_visitor_visit_addressof_expression ((ValaCodeVisitor*) codegen, self);
	vala_code_visitor_visit_expression ((ValaCodeVisitor*) codegen, (ValaExpression*) self);
}

static void
vala_addressof_expression_class_init (ValaAddressofExpressionClass * klass,
                                      gpointer klass_data)
{
	vala_addressof_expression_parent_class = g_type_class_peek_parent (klass);
	((ValaCodeNodeClass *) klass)->finalize = vala_addressof_expression_finalize;
	g_type_class_adjust_private_offset (klass, &ValaAddressofExpression_private_offset);
	((ValaCodeNodeClass *) klass)->accept = (void (*) (ValaCodeNode*, ValaCodeVisitor*)) vala_addressof_expression_real_accept;
	((ValaCodeNodeClass *) klass)->accept_children = (void (*) (ValaCodeNode*, ValaCodeVisitor*)) vala_addressof_expression_real_accept_children;
	((ValaCodeNodeClass *) klass)->to_string = (gchar* (*) (ValaCodeNode*)) vala_addressof_expression_real_to_string;
	((ValaCodeNodeClass *) klass)->replace_expression = (void (*) (ValaCodeNode*, ValaExpression*, ValaExpression*)) vala_addressof_expression_real_replace_expression;
	((ValaExpressionClass *) klass)->is_pure = (gboolean (*) (ValaExpression*)) vala_addressof_expression_real_is_pure;
	((ValaExpressionClass *) klass)->is_accessible = (gboolean (*) (ValaExpression*, ValaSymbol*)) vala_addressof_expression_real_is_accessible;
	((ValaExpressionClass *) klass)->is_non_null = (gboolean (*) (ValaExpression*)) vala_addressof_expression_real_is_non_null;
	((ValaCodeNodeClass *) klass)->check = (gboolean (*) (ValaCodeNode*, ValaCodeContext*)) vala_addressof_expression_real_check;
	((ValaCodeNodeClass *) klass)->emit = (void (*) (ValaCodeNode*, ValaCodeGenerator*)) vala_addressof_expression_real_emit;
}

static void
vala_addressof_expression_instance_init (ValaAddressofExpression * self,
                                         gpointer klass)
{
	self->priv = vala_addressof_expression_get_instance_private (self);
}

static void
vala_addressof_expression_finalize (ValaCodeNode * obj)
{
	ValaAddressofExpression * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALA_TYPE_ADDRESSOF_EXPRESSION, ValaAddressofExpression);
	_vala_code_node_unref0 (self->priv->_inner);
	VALA_CODE_NODE_CLASS (vala_addressof_expression_parent_class)->finalize (obj);
}

/**
 * Represents an address-of expression.
 *
 * {{{ &foo }}}
 */
static GType
vala_addressof_expression_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValaAddressofExpressionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_addressof_expression_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaAddressofExpression), 0, (GInstanceInitFunc) vala_addressof_expression_instance_init, NULL };
	GType vala_addressof_expression_type_id;
	vala_addressof_expression_type_id = g_type_register_static (VALA_TYPE_EXPRESSION, "ValaAddressofExpression", &g_define_type_info, 0);
	ValaAddressofExpression_private_offset = g_type_add_instance_private (vala_addressof_expression_type_id, sizeof (ValaAddressofExpressionPrivate));
	return vala_addressof_expression_type_id;
}

GType
vala_addressof_expression_get_type (void)
{
	static volatile gsize vala_addressof_expression_type_id__once = 0;
	if (g_once_init_enter (&vala_addressof_expression_type_id__once)) {
		GType vala_addressof_expression_type_id;
		vala_addressof_expression_type_id = vala_addressof_expression_get_type_once ();
		g_once_init_leave (&vala_addressof_expression_type_id__once, vala_addressof_expression_type_id);
	}
	return vala_addressof_expression_type_id__once;
}

